#include <boarddefs.h>
#include <IRremote.h>
#include <IRremoteInt.h>
#include <ir_Lego_PF_BitStreamEncoder.h>

#include <nRF24L01.h>
#include <printf.h>
#include <RF24.h>
#include <RF24_config.h>
#include "EEPROM.h"

#define IR_SONY         1
#define IR_RC5          2
#define IR_RC6          3
#define IR_NEC          4
#define IR_PANASONIC    5
#define IR_JVC          6
#define IR_SAMSUNG      7
#define IR_WHYNTER      8
#define IR_AIWA_RC_T501 9
#define IR_LG           10
#define IR_SANYO        11
#define IR_MITSUBISHI   12
#define IR_DISH         13
#define IR_SHARP        14
#define IR_DENON        15
#define ADDRESS_PROGRAM_MODE    0x353535
#define BUTTON_TYPE_MODE        0x12120000

#define LISTENING                 0
#define ADDRESS_PROGRAM_RECEIVED  1
#define BUTTON_TYPE_RECEIVED      2

#define IR_PIN 3

RF24 radio(7, 8);
IRsend irsend;

uint32_t data;
bool newData = false;

uint8_t data_buffer[32];
uint8_t payloadSize;

volatile uint8_t state = 0;

uint8_t buttonType = 0;
uint8_t buttonBits = 0;

unsigned int eepromDataAddress = 100;
volatile uint32_t eepromData = 0;
volatile uint32_t address = 0xC2C2BB;

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  printf_begin();

  Serial.println("Starting up...");
 
  /*
    //Clear EEPROM
    for (int i = 0; i <= 4; i++)  {
      EepromWrite(eepromDataAddress+i, (unsigned char)0);
      delay(250);
    }
*/
  //Check EEPROM for stored address
  uint8_t eepromDataByte;
  for (int i = 0; i <= 4; i++)  {
    eepromData = ((eepromData << 8) | EepromRead(eepromDataAddress + i));
    delay(250);
  }

  if (eepromData != 0)  {
    address = eepromData;
  }
  Serial.println(address, HEX);

  radio.begin();

  radio.enableDynamicPayloads();
  radio.setCRCLength(RF24_CRC_8);
  radio.setChannel(9);
  radio.setDataRate(RF24_250KBPS);
  radio.setAddressWidth(3);

  radio.openReadingPipe(0, address);
  radio.startListening();

}

void loop() {
  // put your main code here, to run repeatedly:
  getData();
  stateMachine();

}

void getData()  {
  if (radio.available())  {
    payloadSize = radio.getDynamicPayloadSize();
    radio.read(&data_buffer, payloadSize);
    newData = true;

    data = 0;
    for (int i = payloadSize - 1; i >= 0; i--)  {
      data = (data << 8 | data_buffer[i]);
    }

    for (int i = 0; i < payloadSize; i++)  {
      data_buffer[i] = 0;
    }

    if(!data)  {
      newData = false; 
      Serial.println(data, HEX);
    }

    
  }
}

void stateMachine()  {
  uint8_t currentState = state;

  switch (currentState)  {
    case LISTENING :
      if (newData)  {
        if (data == ADDRESS_PROGRAM_MODE)  {
          //Serial.println("program mode!");
          state = ADDRESS_PROGRAM_RECEIVED;
          //Serial.println(state);
        }
        else if ((data & BUTTON_TYPE_MODE) == BUTTON_TYPE_MODE)  {
          buttonType = (data & 0xFFFF) >> 8;
          buttonBits = data & 0xFF;
          //Serial.print("Bits: ");
          //Serial.println(buttonBits);
          //Serial.print("Type: ");
          //Serial.println(buttonType);
          state = BUTTON_TYPE_RECEIVED;
        }
        newData = false;
      }
      break;
    case ADDRESS_PROGRAM_RECEIVED :
      if (newData)  {
        Serial.println("programming...");
        radio.stopListening();
        radio.openReadingPipe(0, data);
        radio.startListening();

        radio.printDetails();
        //Write back address to Eeprom...leave commented out for testing
        /* uint32_t temp = data;
          for(int i = 4; i > 0; i--)   {
           EepromWrite(eepromDataAddress + i, (uint8_t)(temp & 0xFF));
           temp = temp >> 8;
           delay(250);
          }*/

        newData = false;
        state = LISTENING;
      }
      break;
    case BUTTON_TYPE_RECEIVED :
      if (newData)  {
        if (buttonType == IR_SONY)  {
          irsend.sendSony(data, buttonBits);
        }
        else if (buttonType == IR_RC5)  {
          irsend.sendRC5(data, buttonBits);
        }
        else if (buttonType == IR_RC6)  {
          irsend.sendRC6(data, buttonBits);
        }
        else  {
          Serial.println("sending NEC");
          irsend.sendNEC(data, buttonBits);
        }

        newData = false;
        state = LISTENING;
      }
      break;
    default :
      state = LISTENING;
  }
}

