#include "EEPROM.h"

/**
 * EEPROM register map
 */
#define EEARH_ADDR (unsigned char *) 0x42
#define EEARL_ADDR (unsigned char *) 0x41
#define EEDR_ADDR (unsigned char *) 0x40
#define EECR_ADDR (unsigned char *) 0x3F

#define EECR_EEPE_MASK 0x02
#define EECR_EEMPE_MASK 0x04
#define EECR_EERE_MASK 0x01

volatile uint8_t* addEEARH = (uint8_t*)0x0042;
volatile uint8_t* addEEARL = (uint8_t*)0x0041;
volatile uint8_t* addEEDR = (uint8_t*)0x0040;
volatile uint8_t* addEECR = (uint8_t*)0x003F;


/**
 * Write a 8-bit data to a specified EEPROM address 
 */
void EepromWrite(unsigned int address, unsigned char data)  {

  *addEECR &= 0xCF;
  
  while(*addEECR & 0x02);

  *addEEARH = (( address >> 8) & 0x03);
  *addEEARL = (address & 0xFF);
  *addEEDR = data;

  //*addEECR = 0x04;
  //*addEECR = 0x06;
   asm volatile("ldi r24, 0x04\n\t"
               "out 0x1f, r24\n\t"
               "ldi r24, 0x06\n\t"
               "out 0x1f, r24\n\t"
               :: );
}

/**
 * Read 8-bit data from a specified EEPROM address
 */
unsigned char EepromRead (unsigned short address)  {

  unsigned char * portEEPROMControlRegister ;
  unsigned char * portEEPROMAddressHighRegister ;
  unsigned char * portEEPROMAddressLowRegister ;
  unsigned char * portEEPROMDataRegister ;
  unsigned char data ;

  portEEPROMControlRegister = EECR_ADDR;
  portEEPROMAddressHighRegister = EEARH_ADDR;
  portEEPROMAddressLowRegister = EEARL_ADDR;
  portEEPROMDataRegister = EEDR_ADDR;

  /* Wait for completion of previous write */
  while (* portEEPROMControlRegister & EECR_EEPE_MASK );

  /* Set up address register */
  *portEEPROMAddressHighRegister = (( address >> 8) & 0x03);
  *portEEPROMAddressLowRegister = (address & 0xFF);

  /* Start eeprom read by writing EERE */
  *portEEPROMControlRegister |= EECR_EERE_MASK;

  /* Return data from Data Register */
  data = *portEEPROMDataRegister;

return data ;
}
